"""
Type annotations for botocore.configloader module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

def multi_file_load_config(*filenames: str) -> dict[str, Any]: ...
def load_config(config_filename: str) -> dict[str, Any]: ...
def raw_config_parse(config_filename: str, parse_subsections: bool = ...) -> dict[str, Any]: ...
def build_profile_map(parsed_ini_config: dict[str, Any]) -> dict[str, Any]: ...
