"""
Type annotations for botocore.history module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any

logger: Logger = ...

HISTORY_RECORDER: HistoryRecorder | None

class BaseHistoryHandler:
    def emit(self, event_type: str, payload: Any, source: str) -> None: ...

class HistoryRecorder:
    def __init__(self) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def add_handler(self, handler: BaseHistoryHandler) -> None: ...
    def record(self, event_type: str, payload: Any, source: str = ...) -> None: ...

def get_global_history_recorder() -> HistoryRecorder: ...
